@@ -4,7 +4,7 @@
 //                            Author 1  : Sai Lai Lo (sll)
 //                            Author 2  : Duncan Grisby (dgrisby)
 //
-//    Copyright (C) 2002-2008 Apasphere Ltd
+//    Copyright (C) 2002-2012 Apasphere Ltd
 //    Copyright (C) 2001 AT&T Laboratories Cambridge
 //
 //    This file is part of the omniORB library
@@ -29,138 +29,6 @@
 //	*** PROPRIETARY INTERFACE ***
 // 
 
-/*
-  $Log$
-  Revision 1.1.4.25  2008/10/24 16:45:29  dgrisby
-  Last socket in poll list incorrectly thought it was still in the list.
-  Thanks Serguei Kolos for the bug report.
-
-  Revision 1.1.4.24  2008/05/29 13:08:35  dgrisby
-  Cope with fd 0 in pipe files. Thanks Wei Jiang.
-
-  Revision 1.1.4.23  2007/12/09 01:35:08  dgrisby
-  Race condition between Peek / select thread when data in buffer.
-
-  Revision 1.1.4.22  2007/07/04 09:13:32  dgrisby
-  Condition variable used in Peek() was leaked.
-
-  Revision 1.1.4.21  2007/06/16 15:12:29  dgrisby
-  Incorrect initialisation if pipe creation failed on VxWorks.
-
-  Revision 1.1.4.20  2007/06/06 17:01:26  dgrisby
-  Fix potential race conditions in SocketCollection set / poll access.
-  Cope with WSAEINVAL as well as WSAENOTSOCK in Windows case.
-
-  Revision 1.1.4.19  2007/02/26 15:27:13  dgrisby
-  Set pipe file descriptors to close on exec.
-
-  Revision 1.1.4.18  2006/12/28 18:24:28  dgrisby
-  Downgrade assertion about failing to find all readable sockets to be a
-  warning.
-
-  Revision 1.1.4.17  2006/11/16 14:07:40  dgrisby
-  Peek failed to clear pd_fd_index, which could lead to a race condition
-  where the wrong socket was removed from the poll list.
-
-  Revision 1.1.4.16  2006/05/16 13:32:04  dgrisby
-  Silly bug in setting Peek timeouts with conflicting Peeks.
-
-  Revision 1.1.4.15  2006/05/15 17:11:15  dgrisby
-  Limit select timeout in case time goes backwards.
-
-  Revision 1.1.4.14  2006/05/02 13:07:13  dgrisby
-  Idle giopMonitor SocketCollections would not exit at shutdown.
-
-  Revision 1.1.4.13  2006/01/19 16:05:02  dgrisby
-  Windows build fixes.
-
-  Revision 1.1.4.12  2005/11/19 17:33:28  dgrisby
-  Previous fix for the race condition was stupidly commented out...
-
-  Revision 1.1.4.11  2005/11/18 18:25:57  dgrisby
-  Race condition between connection deletion and Select.
-
-  Revision 1.1.4.10  2005/08/03 09:43:13  dgrisby
-  setSelectable could cause a readable socket to be handled twice.
-
-  Revision 1.1.4.9  2005/08/02 09:42:53  dgrisby
-  Two threads could be dispatched for one call, one by Peek, one by Select.
-
-  Revision 1.1.4.8  2005/06/24 14:31:30  dgrisby
-  Allow multiple threads to Peek() without clashing. Not yet tested on
-  Windows.
-
-  Revision 1.1.4.7  2005/04/26 17:08:06  dgrisby
-  Handle all error returns from poll().
-
-  Revision 1.1.4.6  2005/03/02 12:10:50  dgrisby
-  setSelectable / Peek fixes.
-
-  Revision 1.1.4.5  2005/01/25 16:43:37  dgrisby
-  Properly handle data_in_buffer indicator.
-
-  Revision 1.1.4.4  2005/01/17 14:46:19  dgrisby
-  Windows SocketCollection implementation.
-
-  Revision 1.1.4.3  2005/01/13 21:09:59  dgrisby
-  New SocketCollection implementation, using poll() where available and
-  select() otherwise. Windows specific version to follow.
-
-  Revision 1.1.4.2  2005/01/06 23:10:12  dgrisby
-  Big merge from omni4_0_develop.
-
-  Revision 1.1.4.1  2003/03/23 21:02:31  dgrisby
-  Start of omniORB 4.1.x development branch.
-
-  Revision 1.1.2.14  2003/02/17 10:39:52  dgrisby
-  Fix inevitable Windows problem.
-
-  Revision 1.1.2.13  2003/02/17 01:46:23  dgrisby
-  Pipe to kick select thread (on Unix).
-
-  Revision 1.1.2.12  2003/01/28 12:17:09  dgrisby
-  Bug with Select() ignoring data in buffer indications.
-
-  Revision 1.1.2.11  2002/10/14 15:27:41  dgrisby
-  Typo in fcntl error check.
-
-  Revision 1.1.2.10  2002/08/21 06:23:15  dgrisby
-  Properly clean up bidir connections and ropes. Other small tweaks.
-
-  Revision 1.1.2.9  2002/03/18 16:50:18  dpg1
-  New threadPoolWatchConnection parameter.
-
-  Revision 1.1.2.8  2002/03/14 12:21:49  dpg1
-  Undo accidental scavenger period change, remove invalid assertion.
-
-  Revision 1.1.2.7  2002/03/13 16:05:38  dpg1
-  Transport shutdown fixes. Reference count SocketCollections to avoid
-  connections using them after they are deleted. Properly close
-  connections when in thread pool mode.
-
-  Revision 1.1.2.6  2002/02/26 14:06:45  dpg1
-  Recent changes broke Windows.
-
-  Revision 1.1.2.5  2002/02/13 16:02:38  dpg1
-  Stability fixes thanks to Bastiaan Bakker, plus threading
-  optimisations inspired by investigating Bastiaan's bug reports.
-
-  Revision 1.1.2.4  2001/08/24 15:56:44  sll
-  Fixed code which made the wrong assumption about the semantics of
-  do { ...; continue; } while(0)
-
-  Revision 1.1.2.3  2001/08/02 13:00:53  sll
-  Do not use select(0,0,0,0,&timeout), it doesn't work on win32.
-
-  Revision 1.1.2.2  2001/08/01 15:56:07  sll
-  Workaround MSVC++ bug. It generates wrong code with FD_ISSET and FD_SET
-  under certain conditions.
-
-  Revision 1.1.2.1  2001/07/31 16:16:26  sll
-  New transport interface to support the monitoring of active connections.
-
-*/
-
 #include <omniORB4/CORBA.h>
 #include <omniORB4/giopEndpoint.h>
 #include <SocketCollection.h>
@@ -192,119 +60,10 @@ OMNI_NAMESPACE_BEGIN(omni)
 
 #define GDB_DEBUG
 
-/////////////////////////////////////////////////////////////////////////
-void
-SocketSetTimeOut(unsigned long abs_sec, unsigned long abs_nsec,
-		 struct timeval& t)
-{
-  if (abs_sec == 0 && abs_nsec == 0) {
-    // Avoid get_time call which is expensive on some platforms.
-    t.tv_sec = t.tv_usec = 0;
-    return;
-  }
-
-  unsigned long now_sec, now_nsec;
-  omni_thread::get_time(&now_sec,&now_nsec);
-
-  if ((abs_sec <= now_sec) && ((abs_sec < now_sec) || (abs_nsec < now_nsec))) {
-    t.tv_sec = t.tv_usec = 0;
-  }
-  else {
-    t.tv_sec = abs_sec - now_sec;
-    if (abs_nsec >= now_nsec) {
-      t.tv_usec = (abs_nsec - now_nsec) / 1000;
-    }
-    else {
-      t.tv_usec = (1000000000 + abs_nsec - now_nsec) / 1000;
-      t.tv_sec -= 1;
-    }
-  }
-}
 
 /////////////////////////////////////////////////////////////////////////
-int
-SocketSetnonblocking(SocketHandle_t sock) {
-# if defined(__vxWorks__)
-  int fl = TRUE;
-  if (ioctl(sock, FIONBIO, (int)&fl) == ERROR) {
-    return RC_INVALID_SOCKET;
-  }
-  return 0;
-# elif defined(__VMS)
-  int fl = 1;
-  if (ioctl(sock, FIONBIO, &fl) == RC_INVALID_SOCKET) {
-    return RC_INVALID_SOCKET;
-  }
-  return 0;
-# elif defined(__WIN32__)
-  u_long v = 1;
-  if (ioctlsocket(sock,FIONBIO,&v) == RC_SOCKET_ERROR) {
-    return RC_INVALID_SOCKET;
-  }
-  return 0;
-# else
-  int fl = O_NONBLOCK;
-  if (fcntl(sock,F_SETFL,fl) == RC_SOCKET_ERROR) {
-    return RC_INVALID_SOCKET;
-  }
-  return 0;
-# endif
-}
-
-/////////////////////////////////////////////////////////////////////////
-int
-SocketSetblocking(SocketHandle_t sock) {
-# if defined(__vxWorks__)
-  int fl = FALSE;
-  if (ioctl(sock, FIONBIO, (int)&fl) == ERROR) {
-    return RC_INVALID_SOCKET;
-  }
-  return 0;
-# elif defined(__VMS)
-  int fl = 0;
-  if (ioctl(sock, FIONBIO, &fl) == RC_INVALID_SOCKET) {
-    return RC_INVALID_SOCKET;
-  }
-  return 0;
-# elif defined(__WIN32__)
-  u_long v = 0;
-  if (ioctlsocket(sock,FIONBIO,&v) == RC_SOCKET_ERROR) {
-    return RC_INVALID_SOCKET;
-  }
-  return 0;
-# else
-  int fl = 0;
-  if (fcntl(sock,F_SETFL,fl) == RC_SOCKET_ERROR) {
-    return RC_INVALID_SOCKET;
-  }
-  return 0;
-# endif
-}
-
-
-/////////////////////////////////////////////////////////////////////////
-int
-SocketSetCloseOnExec(SocketHandle_t sock) {
-# if defined(__vxWorks__) || defined(__ETS_KERNEL__)
-  // Not supported on vxWorks or ETS
-  return 0;
-# elif defined(__WIN32__)
-  SetHandleInformation((HANDLE)sock, HANDLE_FLAG_INHERIT, 0);
-  return 0;
-# else
-  int fl = FD_CLOEXEC;
-  if (fcntl(sock,F_SETFD,fl) == RC_SOCKET_ERROR) {
-    return RC_INVALID_SOCKET;
-  }
-  return 0;
-# endif
-}
-
-
-/////////////////////////////////////////////////////////////////////////
-unsigned long SocketCollection::scan_interval_sec  = 0;
-unsigned long SocketCollection::scan_interval_nsec = 50*1000*1000;
-unsigned      SocketCollection::idle_scans         = 20;
+omni_time_t  SocketCollection::scan_interval(0, 50*1000*1000);
+unsigned int SocketCollection::idle_scans = 20;
 
 /////////////////////////////////////////////////////////////////////////
 // pipe creation
@@ -338,8 +97,8 @@ static void initPipe(int& pipe_read, int& pipe_write)
     pipe_read  = filedes[0];
     pipe_write = filedes[1];
 
-    SocketSetCloseOnExec(pipe_read);
-    SocketSetCloseOnExec(pipe_write);
+    tcpSocket::setCloseOnExec(pipe_read);
+    tcpSocket::setCloseOnExec(pipe_write);
   }
   else {
     omniORB::logs(5, "Unable to create pipe for SocketCollection.");
@@ -370,7 +129,7 @@ static void closePipe(int pipe_read, int pipe_write)
 
 SocketCollection::SocketCollection()
   : pd_refcount(1),
-    pd_abs_sec(0), pd_abs_nsec(0),
+    pd_collection_lock("SocketCollection::pd_collection_lock"),
     pd_pipe_full(0),
     pd_idle_count(idle_scans),
     pd_pollfd_n(0),
@@ -429,19 +188,18 @@ SocketCollection::Select() {
   int count;
   unsigned index, pollfd_n;
 
-  // (pd_abs_sec,pd_abs_nsec) defines the absolute time at which we
-  // process the socket list.
-  SocketSetTimeOut(pd_abs_sec,pd_abs_nsec,timeout);
+  // pd_abs_time defines the absolute time at which we process the
+  // socket list.
+  tcpSocket::setTimeout(pd_abs_time, timeout);
 
   if ((timeout.tv_sec == 0 && timeout.tv_usec == 0) ||
-      timeout.tv_sec > scan_interval_sec) {
+      timeout.tv_sec > scan_interval.s) {
 
     // Time to scan the socket list...
 
-    omni_thread::get_time(&pd_abs_sec,&pd_abs_nsec,
-			  scan_interval_sec,scan_interval_nsec);
-    timeout.tv_sec  = scan_interval_sec;
-    timeout.tv_usec = scan_interval_nsec / 1000;
+    omni_thread::get_time(pd_abs_time, scan_interval);
+    timeout.tv_sec  = scan_interval.s;
+    timeout.tv_usec = scan_interval.ns / 1000;
 
     index = 0;
     {
@@ -579,7 +337,7 @@ SocketCollection::Select() {
 	else {
 	  // Force a list scan next time
 	  pd_changed = 1;
-	  pd_abs_sec = pd_abs_nsec = 0;
+	  pd_abs_time.assign(0,0);
 	}
       }
       index++;
@@ -607,7 +365,7 @@ SocketCollection::Select() {
 
       // Force a list scan next time
       pd_changed = 1;
-      pd_abs_sec = pd_abs_nsec = 0;
+      pd_abs_time.assign(0,0);
     }
     else if (ERRNO != RC_EINTR) {
       if (omniORB::trace(20)) {
@@ -665,7 +423,7 @@ SocketHolder::setSelectable(int            now,
       // We must not grow the poll lists here, since the Select thread
       // is possibly accessing pd_pollfds outside the lock. Instead,
       // we ensure the Select thread rescans next iteration.
-      pd_belong_to->pd_abs_sec = pd_belong_to->pd_abs_nsec = 0;
+      pd_belong_to->pd_abs_time.assign(0,0);
     }
   }
 
@@ -676,7 +434,7 @@ SocketHolder::setSelectable(int            now,
 
   if (pd_data_in_buffer) {
     // Force Select() to scan through the connections right away
-    pd_belong_to->pd_abs_sec = pd_belong_to->pd_abs_nsec = 0;
+    pd_belong_to->pd_abs_time.assign(0,0);
   }
 
 #ifdef UnixArchitecture
@@ -726,8 +484,7 @@ SocketHolder::Peek()
   {
     omni_tracedmutex_lock l(pd_belong_to->pd_collection_lock);
 
-    unsigned long s, ns;
-    s = ns = 0;
+    omni_time_t deadline;
 
     while (1) {
       if (pd_selectable && !pd_peeking) {
@@ -742,16 +499,14 @@ SocketHolder::Peek()
 	}
 	if (!pd_peek_cond)
 	  pd_peek_cond =
-	    new omni_tracedcondition(&pd_belong_to->pd_collection_lock);
+	    new omni_tracedcondition(&pd_belong_to->pd_collection_lock,
+				     "SocketHolder::pd_peek_cond");
 	
-	if (s == 0 && ns == 0) {
+	if (!deadline) {
 	  // Set timeout for condition wait
-	  unsigned long rs, rns;
-	  rs  = SocketCollection::scan_interval_sec;
-	  rns = SocketCollection::scan_interval_nsec;
-	  omni_thread::get_time(&s, &ns, rs, rns);
+	  omni_thread::get_time(deadline, SocketCollection::scan_interval);
 	}
-	int signalled = pd_peek_cond->timedwait(s, ns);
+	int signalled = pd_peek_cond->timedwait(deadline);
 	
 	if (pd_selectable && !pd_peeking) {
 	  // OK to go ahead and peek
@@ -774,8 +529,8 @@ SocketHolder::Peek()
     pd_peek_go = 0;
   }
 
-  int timeout = (SocketCollection::scan_interval_sec * 1000 +
-		 SocketCollection::scan_interval_nsec / 1000000);
+  int timeout = (SocketCollection::scan_interval.s * 1000 +
+		 SocketCollection::scan_interval.ns / 1000000);
 
   pollfd pfd;
   pfd.fd = pd_socket;
@@ -852,8 +607,8 @@ SocketHolder::Peek()
 
 SocketCollection::SocketCollection()
   : pd_refcount(1),
-    pd_select_cond(&pd_collection_lock),
-    pd_abs_sec(0), pd_abs_nsec(0),
+    pd_collection_lock("SocketCollection::pd_collection_lock"),
+    pd_select_cond(&pd_collection_lock, "SocketCollection::pd_select_cond"),
     pd_pipe_full(0),
     pd_idle_count(idle_scans),
     pd_collection(0),
@@ -880,21 +635,20 @@ SocketCollection::Select() {
   int count;
   unsigned index;
 
-  // (pd_abs_sec,pd_abs_nsec) defines the absolute time at which we
-  // process the socket list.
-  SocketSetTimeOut(pd_abs_sec,pd_abs_nsec,timeout);
+  // pd_abs_time defines the absolute time at which we process the
+  // socket list.
+  tcpSocket::setTimeout(pd_abs_time, timeout);
 
   fd_set rfds;
   
   if ((timeout.tv_sec == 0 && timeout.tv_usec == 0) ||
-      timeout.tv_sec > scan_interval_sec) {
+      timeout.tv_sec > scan_interval.s) {
 
     // Time to scan the socket list...
 
-    omni_thread::get_time(&pd_abs_sec,&pd_abs_nsec,
-			  scan_interval_sec,scan_interval_nsec);
-    timeout.tv_sec  = scan_interval_sec;
-    timeout.tv_usec = scan_interval_nsec / 1000;
+    omni_thread::get_time(pd_abs_time, scan_interval);
+    timeout.tv_sec  = scan_interval.s;
+    timeout.tv_usec = scan_interval.ns / 1000;
 
     {
       omni_tracedmutex_lock sync(pd_collection_lock);
@@ -942,7 +696,7 @@ SocketCollection::Select() {
     // Windows doesn't let us select on an empty fd_set, so we sleep
     // on a condition variable instead.
     omni_tracedmutex_lock sync(pd_collection_lock);
-    pd_select_cond.timedwait(pd_abs_sec,pd_abs_nsec);
+    pd_select_cond.timedwait(pd_abs_time);
     count = 0;
   }
 
@@ -1006,7 +760,7 @@ SocketCollection::Select() {
 
       // Force a list scan next time
       pd_changed = 1;
-      pd_abs_sec = pd_abs_nsec = 0;
+      pd_abs_time.assign(0,0);
     }
     else if (err != RC_EINTR) {
       if (omniORB::trace(1)) {
@@ -1064,7 +818,7 @@ SocketHolder::setSelectable(int            now,
 
   if (pd_data_in_buffer) {
     // Force Select() to scan through the connections right away
-    pd_belong_to->pd_abs_sec = pd_belong_to->pd_abs_nsec = 0;
+    pd_belong_to->pd_abs_time.assign(0,0);
   }
 
   // Wake up a thread waiting to peek, if there is one
@@ -1088,8 +842,7 @@ SocketHolder::Peek()
   {
     omni_tracedmutex_lock l(pd_belong_to->pd_collection_lock);
 
-    unsigned long s, ns;
-    s = ns = 0;
+    omni_time_t deadline;
 
     while (1) {
       if (pd_selectable && !pd_peeking) {
@@ -1104,16 +857,14 @@ SocketHolder::Peek()
 	}
 	if (!pd_peek_cond)
 	  pd_peek_cond =
-	    new omni_tracedcondition(&pd_belong_to->pd_collection_lock);
+	    new omni_tracedcondition(&pd_belong_to->pd_collection_lock,
+				     "SocketHolder::pd_peek_cond");
 	
-	if (s == 0 && ns == 0) {
+	if (!deadline) {
 	  // Set timeout for condition wait
-	  unsigned long rs, rns;
-	  rs  = SocketCollection::scan_interval_sec;
-	  rns = SocketCollection::scan_interval_nsec;
-	  omni_thread::get_time(&s, &ns, rs, rns);
+	  omni_thread::get_time(deadline, SocketCollection::scan_interval);
 	}
-	int signalled = pd_peek_cond->timedwait(s, ns);
+	int signalled = pd_peek_cond->timedwait(deadline);
 	
 	if (pd_selectable && !pd_peeking) {
 	  // OK to go ahead and peek
@@ -1137,8 +888,8 @@ SocketHolder::Peek()
   }
 
   struct timeval timeout;
-  timeout.tv_sec  = SocketCollection::scan_interval_sec;
-  timeout.tv_usec = SocketCollection::scan_interval_nsec / 1000;
+  timeout.tv_sec  = SocketCollection::scan_interval.s;
+  timeout.tv_usec = SocketCollection::scan_interval.ns / 1000;
   fd_set rfds;
 
   int retval = -1;
@@ -1200,7 +951,7 @@ SocketHolder::Peek()
 
 SocketCollection::SocketCollection()
   : pd_refcount(1),
-    pd_abs_sec(0), pd_abs_nsec(0),
+    pd_collection_lock("SocketCollection::pd_collection_lock"),
     pd_pipe_full(0),
     pd_idle_count(idle_scans),
     pd_fd_set_n(0),
@@ -1242,21 +993,20 @@ SocketCollection::Select() {
   int count;
   unsigned index;
 
-  // (pd_abs_sec,pd_abs_nsec) defines the absolute time at which we
-  // process the socket list.
-  SocketSetTimeOut(pd_abs_sec,pd_abs_nsec,timeout);
+  // pd_abs_time defines the absolute time at which we process the
+  // socket list.
+  tcpSocket::setTimeout(pd_abs_time, timeout);
 
   fd_set rfds;
 
   if ((timeout.tv_sec == 0 && timeout.tv_usec == 0) ||
-      timeout.tv_sec > scan_interval_sec) {
+      timeout.tv_sec > scan_interval.s) {
 
     // Time to scan the socket list...
 
-    omni_thread::get_time(&pd_abs_sec,&pd_abs_nsec,
-			  scan_interval_sec,scan_interval_nsec);
-    timeout.tv_sec  = scan_interval_sec;
-    timeout.tv_usec = scan_interval_nsec / 1000;
+    omni_thread::get_time(pd_abs_time, scan_interval);
+    timeout.tv_sec  = scan_interval.s;
+    timeout.tv_usec = scan_interval.ns / 1000;
 
     {
       omni_tracedmutex_lock sync(pd_collection_lock);
@@ -1382,7 +1132,7 @@ SocketCollection::Select() {
 
       // Force a list scan next time
       pd_changed = 1;
-      pd_abs_sec = pd_abs_nsec = 0;
+      pd_abs_time.assign(0,0);
     }
     else if (ERRNO != RC_EINTR) {
       if (omniORB::trace(1)) {
@@ -1441,7 +1191,7 @@ SocketHolder::setSelectable(int            now,
 
   if (pd_data_in_buffer) {
     // Force Select() to scan through the connections right away
-    pd_belong_to->pd_abs_sec = pd_belong_to->pd_abs_nsec = 0;
+    pd_belong_to->pd_abs_time.assign(0,0);
   }
 
 #ifdef UnixArchitecture
@@ -1488,8 +1238,7 @@ SocketHolder::Peek()
   {
     omni_tracedmutex_lock l(pd_belong_to->pd_collection_lock);
 
-    unsigned long s, ns;
-    s = ns = 0;
+    omni_time_t deadline;
 
     while (1) {
       if (pd_selectable && !pd_peeking) {
@@ -1504,18 +1253,14 @@ SocketHolder::Peek()
 	}
 	if (!pd_peek_cond)
 	  pd_peek_cond =
-	    new omni_tracedcondition(&pd_belong_to->pd_collection_lock);
+	    new omni_tracedcondition(&pd_belong_to->pd_collection_lock,
+				     "SocketHolder::pd_peek_cond");
 	
-	if (s == 0 && ns == 0) {
+	if (!deadline) {
 	  // Set timeout for condition wait
-	  unsigned long rs, rns;
-	  rs  = SocketCollection::scan_interval_sec  / 2;
-	  rns = SocketCollection::scan_interval_nsec / 2;
-	  if (SocketCollection::scan_interval_sec % 2)
-	    rns += 500000000;
-	  omni_thread::get_time(&s, &ns, rs, rns);
+	  omni_thread::get_time(deadline, SocketCollection::scan_interval);
 	}
-	int signalled = pd_peek_cond->timedwait(s, ns);
+	int signalled = pd_peek_cond->timedwait(deadline);
 	
 	if (pd_selectable && !pd_peeking) {
 	  // OK to go ahead and peek
@@ -1539,9 +1284,8 @@ SocketHolder::Peek()
   }
 
   struct timeval timeout;
-  timeout.tv_sec  = SocketCollection::scan_interval_sec / 2;
-  timeout.tv_usec = SocketCollection::scan_interval_nsec / 1000 / 2;
-  if (SocketCollection::scan_interval_sec % 2) timeout.tv_usec += 500000;
+  timeout.tv_sec  = SocketCollection::scan_interval.s;
+  timeout.tv_usec = SocketCollection::scan_interval.ns / 1000;
   fd_set rfds;
 
   int retval = -1;
@@ -1674,7 +1418,7 @@ SocketCollection::removeSocket(SocketHolder* s)
     
     // Force rescan of sockets ASAP
     pd_changed = 1;
-    pd_abs_sec = pd_abs_nsec = 0;
+    pd_abs_time.assign(0,0);
   }
   if (refcount == 0) delete this;
 }
