@@ -151,7 +151,7 @@ giopImpl11::inputMessageBegin(giopStream* g,
       // This is a GIOP 1.0 message, switch to the implementation of giop 1.0
       // and dispatch again.
       GIOP::Version v = { 1, 0 };
-      ((giopStrand &)*g).version = v;
+      g->strand().version = v;
       g->impl(giopStreamImpl::matchVersion(v));
       OMNIORB_ASSERT(g->impl());
       g->impl()->inputMessageBegin(g,g->impl()->unmarshalWildCardRequestHeader);
@@ -1106,7 +1106,7 @@ giopImpl11::sendMsgErrorMessage(giopStream* g,
   hdr[7] = (char)GIOP::MessageError;
   hdr[8] = hdr[9] = hdr[10] = hdr[11] = 0;
 
-  (void)  g->pd_strand->connection->Send(hdr,12);
+  (void)  g->pd_strand->connection->Send(hdr, 12, g->getDeadline());
 
   g->pd_strand->state(giopStrand::DYING);
 
@@ -1627,23 +1627,33 @@ giopImpl11::copyOutputData(giopStream* g,void* b, size_t sz,
 		  (CORBA::CompletionStatus)g->completion());
   }
 
-  omni::ptr_arith_t newmkr = omni::align_to((omni::ptr_arith_t)g->pd_outb_mkr,
-					    align);
-  OMNIORB_ASSERT(newmkr <= (omni::ptr_arith_t)g->pd_outb_end);
+  omni::ptr_arith_t newmkr = g->outMkr(align);
+  OMNIORB_ASSERT(newmkr <= g->outEnd());
+
+  g->pd_outb_mkr = (void*)newmkr;
 
   if (sz >= giopStream::directSendCutOff) {
 
-    g->pd_outb_mkr = (void*)newmkr;
+    omni::ptr_arith_t outbuf_begin = g->outputBufferStart();
+
+    if (newmkr - outbuf_begin < giopStream::minChunkBeforeDirectSend) {
+      // Copy some of the data into the buffer, to prevent
+      // transmission of a small chunk.
+      size_t current = newmkr - outbuf_begin;
+      size_t avail   = g->outEnd() - newmkr;
+      size_t filler  = giopStream::minChunkBeforeDirectSend - current;
+      if (filler > avail)
+        filler = avail;
+
+      memcpy(g->pd_outb_mkr, b, filler);
+      sz -= filler;
+      g->pd_outb_mkr = (void*)(g->outMkr() + filler);
+      b = (void*)((omni::ptr_arith_t)b + filler);
+      newmkr = g->outMkr();
+    }
 
     if (!g->outputFragmentSize()) {
-
-      omni::ptr_arith_t outbuf_begin = ((omni::ptr_arith_t) 
-					g->pd_currentOutputBuffer + 
-					g->pd_currentOutputBuffer->start);
-
-      CORBA::ULong fsz = (omni::ptr_arith_t) g->pd_outb_mkr - 
-            	         (omni::ptr_arith_t) outbuf_begin + sz - 12;
-
+      CORBA::ULong fsz = g->outMkr() - outbuf_begin + sz - 12;
       *((CORBA::ULong*)((omni::ptr_arith_t)outbuf_begin + 8)) = fsz;
     }
 
@@ -1651,26 +1661,34 @@ giopImpl11::copyOutputData(giopStream* g,void* b, size_t sz,
 
     g->sendCopyChunk(b,sz);
 
+    // After this vector of bytes is sent, the stream may or may not be
+    // 8 bytes aligned. But our output buffer is now emptied and hence
+    // is 8 bytes aligned. Since we sent the whole vector out, we have
+    // to make sure that the next byte will be marshalled at the correct
+    // alignment by adjusting the start of the currentOutputBuffer.
+
     if (g->outputFragmentSize()) {
       size_t leftover = (newmkr + sz) % 0x7;
       if (leftover) {
 	g->pd_currentOutputBuffer->start += leftover;
-	g->pd_outb_mkr = (void*) ((omni::ptr_arith_t) 
-				  g->pd_currentOutputBuffer + 
-				  g->pd_currentOutputBuffer->start);
+	g->pd_outb_mkr = (void*)g->outputBufferStart();
       }
     }
   }
   else {
-    g->pd_outb_mkr = (void*)newmkr;
     while (sz) {
-      size_t avail = (omni::ptr_arith_t) g->pd_outb_end	- 
-	             (omni::ptr_arith_t) g->pd_outb_mkr;
-      if (avail > sz) avail = sz;
+      size_t avail = g->outEnd() - g->outMkr();
+
+      if (avail > sz)
+        avail = sz;
+
       memcpy(g->pd_outb_mkr,b,avail);
       sz -= avail;
-      g->pd_outb_mkr = (void*)((omni::ptr_arith_t) g->pd_outb_mkr + avail);
+
+      g->pd_outb_mkr = (void*)(g->outMkr() + avail);
+
       b = (void*)((omni::ptr_arith_t) b + avail);
+
       if (g->pd_outb_mkr == g->pd_outb_end) {
 	outputFlush(g);
 	if (sz && outputHasReachedLimit(g)) {
@@ -1691,17 +1709,7 @@ giopImpl11::currentOutputPtr(const giopStream* g) {
                      ((omni::ptr_arith_t) g->pd_currentOutputBuffer + 
 		      g->pd_currentOutputBuffer->start);
 
-  CORBA::Long msz = g->outputMessageSize();
-
-  if (msz && g->outputFragmentSize()) {
-    // Message is not being fragmented. Current pointer is just the
-    // current size plus the size sent so far.
-    return msz + fsz;
-  }
-  else {
-    // Pointer is the size sent so far minux the 12 byte GIOP header.
-    return msz + fsz - 12;
-  }
+  return g->outputMessageSize() + fsz - 12;
 }
 
 
