@@ -27,7 +27,7 @@
 
 #include "omniZIOPImpl.h"
 #include <omniORB4/omniInterceptors.h>
-#include <omniORB4/messaging.hh>
+#include <omniORB4/messaging_policy.hh>
 #include <initialiser.h>
 #include <initRefs.h>
 #include <giopStrandFlags.h>
@@ -98,7 +98,7 @@ decodePolicies(const CORBA::PolicyList&            policies,
     CORBA::Policy* policy = policies[index];
 
     switch (policy->policy_type()) {
-    case ZIOP::COMPRESSION_ENABLING_POLICY_ID:
+    case 64: // ZIOP::COMPRESSION_ENABLING_POLICY_ID:
       {
         ZIOP::CompressionEnablingPolicy_var ep =
           ZIOP::CompressionEnablingPolicy::_narrow(policy);
@@ -114,7 +114,7 @@ decodePolicies(const CORBA::PolicyList&            policies,
         break;
       }
 
-    case ZIOP::COMPRESSOR_ID_LEVEL_LIST_POLICY_ID:
+    case 65: // ZIOP::COMPRESSOR_ID_LEVEL_LIST_POLICY_ID:
       {
         ZIOP::CompressionIdLevelListPolicy_var ep =
           ZIOP::CompressionIdLevelListPolicy::_narrow(policy);
@@ -131,7 +131,7 @@ decodePolicies(const CORBA::PolicyList&            policies,
         break;
       }
 
-    case ZIOP::COMPRESSION_LOW_VALUE_POLICY_ID:
+    case 66: // ZIOP::COMPRESSION_LOW_VALUE_POLICY_ID:
       {
         ZIOP::CompressionLowValuePolicy_var ep =
           ZIOP::CompressionLowValuePolicy::_narrow(policy);
@@ -146,7 +146,7 @@ decodePolicies(const CORBA::PolicyList&            policies,
         break;
       }
 
-    case ZIOP::COMPRESSION_MIN_RATIO_POLICY_ID:
+    case 67: // ZIOP::COMPRESSION_MIN_RATIO_POLICY_ID:
       {
         ZIOP::CompressionMinRatioPolicy_var ep =
           ZIOP::CompressionMinRatioPolicy::_narrow(policy);
@@ -215,7 +215,6 @@ setPoliciesInProfile(const CORBA::PolicyList*       policies,
 }
 
 
-
 CORBA::Object_ptr
 omniZIOP::
 setServerPolicies(CORBA::Object_ptr obj, const CORBA::PolicyList& policies)
@@ -1007,13 +1006,13 @@ serverReceiveRequestInterceptor(omniInterceptors::
       // Check serverTransportRule to see if we should allow ziop from
       // this client.
       {
-	transportRules::sequenceString actions;
-	CORBA::ULong   matched_rule;
-	CORBA::Boolean accept_ziop;
-	CORBA::Boolean dump_rule = 0;
-	const char*    why;
+	CORBA::StringSeq actions;
+	CORBA::ULong     matched_rule;
+	CORBA::Boolean   accept_ziop;
+	CORBA::Boolean   dump_rule = 0;
+	const char*      why;
 
-        transportRules& rules = transportRules::serverRules();
+        transportRules&  rules = transportRules::serverRules();
 
 	if ((accept_ziop = rules.match(strand.connection->peeraddress(),
                                        actions, matched_rule))) {
